/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2016 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*     Internet: segger.com   Support: support_embos@segger.com       *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.16                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : SEGGER_RTT_Conf.h
Purpose : Implementation of SEGGER real-time transfer (RTT) which
          allows real-time communication on targets which support
          debugger memory accesses while the CPU is running.
---------------------------END-OF-HEADER------------------------------
*/

#ifndef SEGGER_RTT_CONF_H
#define SEGGER_RTT_CONF_H

#ifdef __ICCARM__
  #include <intrinsics.h>
#endif

/*********************************************************************
*
*       Defines, configurable
*
**********************************************************************
*/

#define SEGGER_RTT_MAX_NUM_UP_BUFFERS             (2)     // Max. number of up-buffers (T->H) available on this target    (Default: 2)
#define SEGGER_RTT_MAX_NUM_DOWN_BUFFERS           (2)     // Max. number of down-buffers (H->T) available on this target  (Default: 2)

#define BUFFER_SIZE_UP                            (1024)  // Size of the buffer for terminal output of target, up to host (Default: 1k)
#define BUFFER_SIZE_DOWN                          (16)    // Size of the buffer for terminal input to target from host (Usually keyboard input) (Default: 16)

#define SEGGER_RTT_PRINTF_BUFFER_SIZE             (64u)    // Size of buffer for RTT printf to bulk-send chars via RTT     (Default: 64)

#define SEGGER_RTT_MODE_DEFAULT                   SEGGER_RTT_MODE_NO_BLOCK_SKIP // Mode for pre-initialized terminal channel (buffer 0)

//
// Target is not allowed to perform other RTT operations while string still has not been stored completely.
// Otherwise we would probably end up with a mixed string in the buffer.
// If using  RTT from within interrupts, multiple tasks or multi processors, define the SEGGER_RTT_LOCK() and SEGGER_RTT_UNLOCK() function here.
//
/*********************************************************************
*
*       RTT lock configuration for SEGGER Embedded Studio,
*       Rowley CrossStudio and GCC
*/
#if (defined __SES_ARM) || (defined __CROSSWORKS_ARM) || (defined __GNUC__)
  #ifdef __ARM_ARCH_6M__
    #define SEGGER_RTT_LOCK() {                                             \
                                 unsigned int LockState;                    \
                                 __asm volatile ("mrs  %0, primask   \n\t"  \
                                                 "mov  r1, $1        \n\t"  \
                                                 "msr  primask, r1   \n\t"  \
                                                 : "=r" (LockState)         \
                                                 :                          \
                                                 : "r1"                     \
                                                );

    #define SEGGER_RTT_UNLOCK()  __asm volatile ("msr   primask, %0  \n\t"  \
                                                 :                          \
                                                 : "r" (LockState)          \
                                                 :                          \
                                                );                          \
                              }
  #elif (defined(__ARM_ARCH_7M__) || defined(__ARM_ARCH_7EM__))
    #define SEGGER_RTT_LOCK() {                                             \
                                 unsigned int LockState;                    \
                                 __asm volatile ("mrs   %0, basepri  \n\t"  \
                                                 "mov   r1, $128     \n\t"  \
                                                 "msr   basepri, r1  \n\t"  \
                                                 : "=r" (LockState)         \
                                                 :                          \
                                                 : "r1"                     \
                                                );

    #define SEGGER_RTT_UNLOCK()  __asm volatile ("msr   basepri, %0  \n\t"  \
                                                 :                          \
                                                 : "r" (LockState)          \
                                                 :                          \
                                                );                          \
                              }
  #elif (defined __ARM7A__)
    #define SEGGER_RTT_LOCK() {                                             \
                                 unsigned int LockState;                    \
                                 __asm volatile ("mrs r1, cpsr       \n\t"  \
                                                 "mov %0, r1         \n\t"  \
                                                 "orr r1, r1, #0xC0  \n\t"  \
                                                 "msr cpsr_c, r1     \n\t"  \
                                                 : "=r" (LockState)         \
                                                 :                          \
                                                 : "r1"                     \
                                                );

    #define SEGGER_RTT_UNLOCK()  __asm volatile ("mov r0, %0         \n\t"  \
                                                 "mrs r1, cpsr       \n\t"  \
                                                 "bic r1, r1, #0xC0  \n\t"  \
                                                 "and r0, r0, #0xC0  \n\t"  \
                                                 "orr r1, r1, r0     \n\t"  \
                                                 "msr cpsr_c, r1     \n\t"  \
                                                 :                          \
                                                 : "r" (LockState)          \
                                                 : "r0","r1"                \
                                                );                          \
                              }
  #endif
#endif

/*********************************************************************
*
*       RTT lock configuration for IAR EWARM
*/
#ifdef __ICCARM__
  #if (defined (__ARM6M__) && (__CORE__ == __ARM6M__))
    #define SEGGER_RTT_LOCK() {                                             \
                                 unsigned int LockState;                    \
                                 LockState = __get_PRIMASK();               \
                                 __set_PRIMASK(1);

    #define SEGGER_RTT_UNLOCK()  __set_PRIMASK(LockState);                  \
                              }
  #elif ((defined (__ARM7EM__) && (__CORE__ == __ARM7EM__)) || (defined (__ARM7M__) && (__CORE__ == __ARM7M__)))
    #define SEGGER_RTT_LOCK() {                                             \
                                 unsigned int LockState;                    \
                                 LockState = __get_BASEPRI();               \
                                 __set_BASEPRI(128);

    #define SEGGER_RTT_UNLOCK()  __set_BASEPRI(LockState);                  \
                              }
  #endif
#endif

/*********************************************************************
*
*       RTT lock configuration for KEIL ARM
*/
#ifdef __CC_ARM
  #if (defined __TARGET_ARCH_6S_M)
    #define SEGGER_RTT_LOCK() {                                                    \
                                 unsigned int LockState;                           \
                                 register unsigned char PRIMASK __asm( "primask"); \
                                 LockState = PRIMASK;                              \
                                 PRIMASK = 1u;                                     \
                                 __schedule_barrier();

    #define SEGGER_RTT_UNLOCK()  PRIMASK = LockState;                              \
                                 __schedule_barrier();                             \
                              }
  #elif (defined(__TARGET_ARCH_7_M) || defined(__TARGET_ARCH_7E_M))
    #define SEGGER_RTT_LOCK() {                                                    \
                                 unsigned int LockState;                           \
                                 register unsigned char BASEPRI __asm( "basepri"); \
                                 LockState = BASEPRI;                              \
                                 BASEPRI = 128u;                                   \
                                 __schedule_barrier();

    #define SEGGER_RTT_UNLOCK()  BASEPRI = LockState;                              \
                                 __schedule_barrier();                             \
                              }
  #endif
#endif

/*********************************************************************
*
*       RTT lock configuration fallback
*/
#ifndef   SEGGER_RTT_LOCK
  #define SEGGER_RTT_LOCK()                // Lock RTT (nestable)   (i.e. disable interrupts)
#endif

#ifndef   SEGGER_RTT_UNLOCK
  #define SEGGER_RTT_UNLOCK()              // Unlock RTT (nestable) (i.e. enable previous interrupt lock state)
#endif

#endif
/*************************** End of file ****************************/
