/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2016 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*     Internet: segger.com   Support: support_embos@segger.com       *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.16                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for STM32F207 (ST MB786)
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"
#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** LED port assignement on MB786 eval board ********************/
#define GPIOG_BASE_ADDR           ((unsigned int)0x40021800)

#define GPIOG_MODER               (*(volatile unsigned int*)(GPIOG_BASE_ADDR + 0x00))
#define GPIOG_ODR                 (*(volatile unsigned int*)(GPIOG_BASE_ADDR + 0x14))
#define GPIOG_BSRR                (*(volatile unsigned int*)(GPIOG_BASE_ADDR + 0x18))

/****** SFRs used for LED-Port **************************************/

#define RCC_BASE_ADDR             ((unsigned int)(0x40023800))
#define RCC_AHB1RSTR              (*(volatile unsigned int*)(RCC_BASE_ADDR + 0x10))
#define RCC_AHB1ENR               (*(volatile unsigned int*)(RCC_BASE_ADDR + 0x30))

#define RCC_LEDPORT_RSTR          RCC_AHB1RSTR
#define RCC_LEDPORT_ENR           RCC_AHB1ENR
#define RCC_LEDPORT_BIT           (6)

/****** Assign LEDs to Ports ****************************************/

#define LED0_BIT        (6)
#define LED1_BIT        (8)
#define LED_MASK_ALL    ((1uL << LED0_BIT) | (1uL << LED1_BIT))

/*********************************************************************
*
*       Typedefs
*
**********************************************************************
*/
typedef void ETH_ISR_HANDLER  (void);
typedef void USB_ISR_HANDLER  (void);

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
static ETH_ISR_HANDLER * _pfEthISRHandler;
static USB_ISR_HANDLER * _pfUSBISRHandler;

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  //
  // Initialize port for LEDs (sample application)
  //
  RCC_LEDPORT_ENR &= ~(1uL << RCC_LEDPORT_BIT);
  RCC_LEDPORT_ENR |=  (1uL << RCC_LEDPORT_BIT);

  GPIOG_MODER &= (3uL << (LED0_BIT * 2)) | (3uL << (LED1_BIT * 2)); // Reset mode, set to input
  GPIOG_MODER |= (1uL << (LED0_BIT * 2)) | (1uL << (LED1_BIT * 2)); // Set to output mode
  GPIOG_BSRR   = (0x10000uL << LED0_BIT) | (0x10000uL << LED1_BIT); // Initially clear LEDs
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with high level on port lines
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    GPIOG_BSRR = (1uL << LED0_BIT);   // Switch on LED0
  } else if (Index == 1) {
    GPIOG_BSRR = (1uL << LED1_BIT);   // Switch on LED1
  }
}

void BSP_ClrLED(int Index) {
  if (Index == 0) {
    GPIOG_BSRR = (0x10000uL << LED0_BIT); // Switch off LED0
  } else if (Index == 1) {
    GPIOG_BSRR = (0x10000uL << LED1_BIT); // Switch off LED1
  }
}

void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    if ((GPIOG_ODR & (1uL << LED0_BIT)) == 0) {  // LED is switched off
      GPIOG_BSRR = (1uL << LED0_BIT);            // Switch on LED0
    } else {
      GPIOG_BSRR = (0x10000uL << LED0_BIT);      // Switch off LED0
    }
  } else if (Index == 1) {
    if ((GPIOG_ODR & (1uL << LED1_BIT)) == 0) {  // LED is switched off
      GPIOG_BSRR = (1uL << LED1_BIT);            // Switch on LED1
    } else {
      GPIOG_BSRR = (0x10000uL << LED1_BIT);      // Switch off LED1
    }
  }
}

/*********************************************************************
*
*       ETH
*
*  Functions for ethernet controllers (as far as present)
*/

/*********************************************************************
*
*       BSP_ETH_ISR_Handler
*/
void BSP_ETH_ISR_Handler(void) {
  OS_EnterInterrupt(); // Inform embOS that interrupt code is running
  if (_pfEthISRHandler) {
    (_pfEthISRHandler)();
  }
  OS_LeaveInterrupt(); // Inform embOS that interrupt code is left
}

/*********************************************************************
*
*       BSP_ETH_InstallISR_Ex()
*/
void BSP_ETH_InstallISR_Ex(int ISRIndex, void (*pfISR)(void), int Prio){
  _pfEthISRHandler = pfISR;
  OS_ARM_InstallISRHandler(ISRIndex, pfISR);  // Ethernet interrupt vector.
  OS_ARM_ISRSetPrio(ISRIndex, Prio);
  OS_ARM_EnableISR(ISRIndex);
}

/*********************************************************************
*
*       BSP_ETH_InstallISR()
*/
void BSP_ETH_InstallISR (void (*pfISR)(void)){
  (void) pfISR;  /* Not implemented, avoid warning */
}

/*********************************************************************
*
*       BSP_ETH_Init()
*
*  Function description
*    This routine is called from the Ethernet driver to initialize
*    port pins enable clocks.
*/
void BSP_ETH_Init (unsigned Unit){

#if 0 // xxx AW: tbd
  unsigned int v;

  (void)  Unit;   /* Multiple Units not supported, avoid warning */
  //
  // Enable I/O clocks
  //
  RCC_APB2ENR |= 0
               | (1 << 5)  // IOPDEN: I/O port D clock enable
               | (1 << 4)  // IOPCEN: I/O port C clock enable
               | (1 << 3)  // IOPBEN: I/O port B clock enable
               | (1 << 2)  // IOPAEN: I/O port A clock enable
               | (1 << 0)  // AFIOEN: Alternate function I/O clock enable
               ;
  v         = AFIO_MAPR;
  v        |=  (1uL << 21); // ETH_REMAP Ethernet MAC I/O remapping
  v        &= ~(1uL << 23); // MII_RMII_SEL MII or RMII selection. Configure Ethernet MAC for connection with an MII PHY
  AFIO_MAPR = v;
  //
  // Set PA2 (ETH_MII_MDIO/ETH_RMII_MDIO) as alternate function output push-pull high-speed (50 MHz)
  //
  v         = GPIOA_CRL;
  v        &= 0xFFFFF0FF;
  v        |= (0xB << 8);
  GPIOA_CRL = v;
  //
  // Set PC1 (ETH_MII_MDC/ETH_RMII_MDC) and PC2 (ETH_MII_TXD2) as alternate function output push-pull high-speed (50 MHz)
  //
  v         = GPIOC_CRL;
  v        &= 0xFFFFF00F;
  v        |= ((0xB << 8) | (0xB << 4));
  GPIOC_CRL = v;
  //
  // Set PB8 (ETH_MII_TXD3), PB11 (ETH_MII_TX_EN/ETH_RMII_TX_EN), PB12 (ETH_MII_TXD0/ETH_RMII_TXD0) and PB13 (ETH_MII_TXD1/ETH_RMII_TXD1) as alternate function output push-pull high-speed (50 MHz)
  //
  v         = GPIOB_CRH;
  v        &= 0xFF000FF0;
  v        |= ((0xB << 20) | (0xB << 16)  | (0xB << 12)  | 0xB);
  GPIOB_CRH = v;
  //
  // Set PA0 (ETH_MII_CRS), PA1 (ETH_MII_RX_CLK/ETH_RMII_REF_CLK), PA3 (ETH_MII_COL) as floating input (reset state)
  //
  v         = GPIOA_CRL;
  v        &= 0xFFFF0F00;
  v        |= ((0x4 << 12) | (0x4 << 4) | 0x4);
  GPIOA_CRL = v;
  //
  // Set PB10 (ETH_MII_RX_ER) as floating input (reset state)
  //
  v         = GPIOB_CRH;
  v        &= 0xFFFFF0FF;
  v        |= (0x4 << 8);
  GPIOB_CRH = v;
  //
  // Set PC3 (ETH_MII_TX_CLK) as floating input (reset state)
  //
  v         = GPIOC_CRL;
  v        &= 0xFFFF0FFF;
  v        |= (0x4 << 12);
  GPIOC_CRL = v;
  //
  // Set PD8 (ETH_RMII_CRS_DV), PD9 (ETH_MII_RXD0/ETH_RMII_RXD0), PD10 (ETH_MII_RXD1/ETH_RMII_RXD1), PD11 (ETH_MII_RXD2) and PD12 (ETH_MII_RXD3) as floating input (reset state)
  //
  v         = GPIOD_CRH;
  v        &= 0xFFF00000;
  v        |= ((0x4 << 16) | (0x4 << 12) | (0x4 << 8) | (0x4 << 4) | 0x4);
  GPIOD_CRH = v;
#else
  (void)  Unit;   /* Avoid warning */
#endif
}

/*********************************************************************
*
*       USB
*
*  Functions for USB controllers (as far as present)
*/

/*********************************************************************
*
*       BSP_USB_ISR_Handler
*/
void BSP_USB_ISR_Handler(void) {
  OS_EnterInterrupt(); // Inform embOS that interrupt code is running
  if (_pfUSBISRHandler) {
    (_pfUSBISRHandler)();
  }
  OS_LeaveInterrupt(); // Inform embOS that interrupt code is left
}

/*********************************************************************
*
*       BSP_USB_InstallISR_Ex()
*/
void BSP_USB_InstallISR_Ex(int ISRIndex, void (*pfISR)(void), int Prio){
  _pfUSBISRHandler = pfISR;
  OS_ARM_InstallISRHandler(ISRIndex, pfISR);  // USB interrupt vector.
  OS_ARM_ISRSetPrio(ISRIndex, Prio);
  OS_ARM_EnableISR(ISRIndex);
}

/****** End of file *************************************************/
