/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2016 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*     Internet: segger.com   Support: support_embos@segger.com       *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.16                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit.c for ST STM32F207 and ST STM32F217

Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS
          Feel free to modify this file acc. to your target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"
#include "SEGGER_SYSVIEW.h"
#ifdef __ICCARM__  // IAR
  #include "exceptions.h"         /* CMSIS compatible irq interface */
#endif

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings (configuration)
*/
#ifndef   OS_FSYS                   /* CPU Main clock frequency     */
  #define OS_FSYS 96000000uL        /* Maximum should be 120 MHz    */
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */
  #define OS_PCLK_TIMER (OS_FSYS)   /* May vary from CPU clock      */
#endif                              /* depending on CPU             */

#ifndef   OS_PCLK_UART              /* Peripheral clock for UART    */
  #define OS_PCLK_UART (OS_FSYS/4)  /* runs on APB1 clock           */
#endif

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ (1000u)
#endif

#ifndef   OS_USE_VARINTTABLE        /* The interrupt vector table   */
  #define OS_USE_VARINTTABLE (0)    /* may be located in RAM        */
#endif

#define   HSE_CLOCK          (25)   /* Clock of external crystal for HSE [MHz] */

#define   RTC_USE_HSE        (0)    /* Set to 1 to feed RTC from HSE clock */

#define OS_Systick SysTick_Handler  /* Map OS systick handler to CMSIS compatible handler name */

#ifndef __ICCARM__  // IAR
  #define __low_level_init SystemInit /* Map IAR __low_level_init to CMSIS compatible function name */
  #ifdef __cplusplus
    extern "C" {
  #endif
  int SystemInit(void);
  #ifdef __cplusplus
    }
  #endif
#endif

/*********************************************************************
*
*       Configuration of communication to embOSView
*/
#ifndef   OS_VIEW_ENABLE            // Global enable of communication
  #define OS_VIEW_ENABLE    (1)     // Default: on
#endif

#ifndef   OS_VIEW_USE_UART          // If set, UART will be used for communication
  #define OS_VIEW_USE_UART  (1)     // Default: 0 => memory access is used
#endif                              // if OS_VIEW_ENABLE is on

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to OSView, please define it to be -1
*       Currently the standard code supports USART 3 and selects
*       USART 3 per default
*/
#ifndef   OS_UART
  #define OS_UART (3)
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE (38400)
#endif

/****** End of configurable options *********************************/

/*********************************************************************
*
*       Check configuration
*
**********************************************************************
*/
#ifndef   DEBUG     /* Should normally be defined as project option */
  #define DEBUG  (0)
#endif

/*********************************************************************
*
*       JLINKMEM and UART settings for embOSView
*
*       Automatically generated from configuration settings
*/
#define OS_USE_JLINKMEM   (OS_VIEW_ENABLE && (OS_VIEW_USE_UART == 0))

#define OS_UART_USED      (OS_VIEW_ENABLE && (OS_VIEW_USE_UART != 0) && (OS_UART == 3))

#if OS_USE_JLINKMEM
  #include "JLINKMEM.h"
#endif

/****** MAP UART initialization function ****************************/

#if  (OS_UART_USED || OS_USE_JLINKMEM)
  #define OS_COM_INIT() OS_COM_Init()
#else
  #define OS_COM_INIT()
#endif

/*********************************************************************
*
*       Vector table
*/
#if (defined __SES_ARM)           // SEGGER Embedded Studio
  extern int _vectors;
  #define __Vectors    _vectors
#elif (defined __CROSSWORKS_ARM)  // Rowley CrossStudio
  extern int _vectors;
  #define __Vectors    _vectors
#elif (defined __ICCARM__)        // IAR
  #define __Vectors    __vector_table
#elif (defined __GNUC__)          // GCC
  extern unsigned char __Vectors;
#elif (defined __CC_ARM)          // KEIL
  extern unsigned char __Vectors;
#endif

/*********************************************************************
*
*       Local defines (sfrs and addresses used in RTOSInit.c)
*
**********************************************************************
*/
#define NVIC_VTOR         (*(volatile OS_U32*) (0xE000ED08uL))
#define NVIC_HFSR         (*(volatile OS_U32*) (0xE000ED2CuL))

/****** RCC, Reset and clock control registers **********************/

#define RCC_BASE_ADDR            (0x40023800uL)

#define RCC_CR                   (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x00))
#define RCC_PLLCFGR              (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x04))
#define RCC_CFGR                 (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x08))
#define RCC_CIR                  (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x0C))
#define RCC_AHB1RSTR             (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x10))
#define RCC_AHB2RSTR             (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x14))
#define RCC_AHB3RSTR             (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x18))
#define RCC_APB1RSTR             (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x20))
#define RCC_APB2RSTR             (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x24))

#define RCC_AHB1ENR              (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x30))
#define RCC_AHB2ENR              (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x34))
#define RCC_AHB3ENR              (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x38))

#define RCC_APB1ENR              (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x40))
#define RCC_APB2ENR              (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x44))

#define RCC_AHB1LPENR            (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x50))
#define RCC_AHB2LPENR            (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x54))
#define RCC_AHB3LPENR            (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x58))

#define RCC_APB1LPENR            (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x60))
#define RCC_APB2LPENR            (*(volatile OS_U32*)(RCC_BASE_ADDR + 0x64))

#define RCC_CR_HSION_BIT         (0)
#define RCC_CR_HSEON_BIT         (16)
#define RCC_CR_HSERDY_BIT        (17)
#define RCC_CR_HSEBYP_BIT        (18)
#define RCC_CR_PLLON_BIT         (24)
#define RCC_CR_PLLRDY_BIT        (25)

#define RCC_CFGR_SW_BIT          (0)
#define RCC_CFGR_SWS_BIT         (2)
#define RCC_CFGR_HPRE_BIT        (4)
#define RCC_CFGR_PPRE1_BIT       (10)
#define RCC_CFGR_PPRE2_BIT       (13)
#define RCC_CFGR_RTCPRE_BIT      (16)

#define RCC_CFGR_PLLM_BIT        (0)
#define RCC_CFGR_PLLN_BIT        (6)
#define RCC_CFGR_PLLP_BIT        (16)
#define RCC_CFGR_PLLQ_BIT        (24)
#define RCC_CFGR_PLLR_BIT        (28)
#define RCC_CFGR_PLLSRC_BIT      (22)


#define PRE1_DIV1                (0x00uL)
#define PRE1_DIV2                (0x04uL)
#define PRE1_DIV4                (0x05uL)
#define PRE1_DIV8                (0x06uL)
#define PRE1_DIV16               (0x07uL)

#define PRE2_DIV1                (0x00uL)
#define PRE2_DIV2                (0x04uL)
#define PRE2_DIV4                (0x05uL)
#define PRE2_DIV8                (0x06uL)
#define PRE2_DIV16               (0x07uL)

/****** SYSCTRL, system controller **********************************/

#define SYS_INT_CTRL_STATE       (*(volatile OS_U32*)(0xE000ED04uL))
#define SYS_PENDSTSET            (26)

#define SYSCTRL_BASE_ADDR        (0x400FE000)
#define SYSCTRL_RCGC1            (*(volatile OS_U32*)(SYSCTRL_BASE_ADDR + 0x104))
#define SYSCTRL_RCGC2            (*(volatile OS_U32*)(SYSCTRL_BASE_ADDR + 0x108))
#define SYSCTRL_RCC              (*(volatile OS_U32*)(SYSCTRL_BASE_ADDR + 0x60))  // Run-mode clock config register
#define SYSCTRL_MISC             (*(volatile OS_U32*)(SYSCTRL_BASE_ADDR + 0x58))  // Interrupt status register
#define SYSCTRL_RIS              (*(volatile OS_U32*)(SYSCTRL_BASE_ADDR + 0x50))  // Raw interrupt status register

#define SYSCTRL_RCC_BYPASS       (0x00000800uL)  // PLL bypass
#define SYSCTRL_RCC_USE_SYSDIV   (0x00400000uL)  // Use sytem clock divider
#define SYSCTRL_RCC_IOSCDIS      (0x00000002uL)  // Internal oscillator disable
#define SYSCTRL_RCC_MOSCDIS      (0x00000001uL)  // Main oscillator disable
#define SYSCTRL_RCC_XTAL_MASK    (0x000003C0uL)  // Crystal attached to main osc
#define SYSCTRL_RCC_OSCSRC_MASK  (0x00000030uL)  // Oscillator input select
#define SYSCTRL_RCC_PWRDN        (0x00002000uL)  // PLL power down
#define SYSCTRL_RCC_OE           (0x00001000uL)  // PLL output enable
#define SYSCTRL_INT_PLL_LOCK     (0x00000040uL)  // PLL lock interrupt
#define SYSCTRL_RCC_SYSDIV_MASK  (0x07800000uL)  // System clock divider

#define SYSPRI1_ADDR             (0xE000ED18uL)

#define SYSHND_CTRL_ADDR         (0xE000ED24uL)  // System Handler Control and State
#define SYSHND_CTRL              (*(volatile OS_U32*) (SYSHND_CTRL_ADDR))

/****** SYS_TICK, used as embOS system timer ************************/

#define SYS_TICK_BASE_ADDR       (0xE000E010uL)
#define SYS_TICK_CONTROL         (*(volatile OS_U32*)(SYS_TICK_BASE_ADDR + 0x00))
#define SYS_TICK_RELOAD          (*(volatile OS_U32*)(SYS_TICK_BASE_ADDR + 0x04))
#define SYS_TICK_VALUE           (*(volatile OS_U32*)(SYS_TICK_BASE_ADDR + 0x08))
#define SYS_TICK_CALIBRATION     (*(volatile OS_U32*)(SYS_TICK_BASE_ADDR + 0x0C))
#define SYS_TICK_ENABLE_BIT      (0)
#define SYS_TICK_INT_ENABLE_BIT  (1)
#define SYS_TICK_CLK_SOURCE_BIT  (2)

/****** NVIC, interrupt controller **********************************/

#define NVIC_SYS_HND_CTRL_MEM    (0x00010000uL)  // Mem manage fault enable
#define NVIC_SYS_HND_CTRL_BUS    (0x00020000uL)  // Bus fault enable
#define NVIC_SYS_HND_CTRL_USAGE  (0x00040000uL)  // Usage fault enable

#define NVIC_BASE_ADDR           (0xE000E000uL)

#define NVIC_PRIOBASE_ADDR       (0xE000E400uL)
#define NVIC_ENABLE_ADDR         (0xE000E100uL)
#define NVIC_DISABLE_ADDR        (0xE000E180uL)
#define NVIC_VTOREG_ADDR         (0xE000ED08uL)

#define NUM_INTERRUPTS           (16+81)

/*****  Interrupt ID numbers used in RTOSint ************************/

#define ISR_ID_NMI               (2)                // NMI fault
#define ISR_ID_HARD              (3)                // Hard fault
#define ISR_ID_MPU               (4)                // MPU fault
#define ISR_ID_BUS               (5)                // Bus fault
#define ISR_ID_USAGE             (6)                // Usage fault
#define ISR_ID_PENDSV            (14)               // PendSV
#define ISR_ID_SYSTICK           (15)               // System Tick
#define ISR_ID_USART1            (53)               // USART1
#define ISR_ID_USART2            (54)               // USART2
#define ISR_ID_USART3            (55)               // USART3

/****** FLASH control register **************************************/

#define FLASH_ACR_BASE_ADDR      (0x40023C00uL)
#define FLASH_ACR                (*(volatile OS_U32*)(FLASH_ACR_BASE_ADDR + 0x00))
#define FLASH_ACR_LATENCY_BIT    (0)
#define FLASH_ACR_PRFTEN_BIT     (8)
#define FLASH_ACR_ICEN_BIT       (9)
#define FLASH_ACR_DCEN_BIT       (10)
#define FLASH_ACR_ICRST_BIT      (11)
#define FLASH_ACR_DCRST_BIT      (12)
#define ACR_LATENCY_MASK         (7uL << FLASH_ACR_LATENCY_BIT)
//
// Flash wait states depend on supply voltage and Cortex-M3 clock
// The following settings are valid for 2.4V <= VCC <= 2.7V
//
#if   (OS_FSYS > 72000000uL)
  #define FLASH_WS (3)
#elif (OS_FSYS > 48000000uL)
  #define FLASH_WS (2)
#elif (OS_FSYS > 24000000uL)
  #define FLASH_WS (1)
#else
  #define FLASH_WS (0)
#endif

/****** OS timer configuration **************************************/

#define OS_TIMER_RELOAD          (OS_PCLK_TIMER / OS_TICK_FREQ - 1)
#if (OS_TIMER_RELOAD >= 0x100000000uL)
  #error "Systick can not be used, please check configuration"
#endif

#define OS_ISR_ID_TICK           ISR_ID_SYSTICK    // We use Sys-Timer

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

#if OS_USE_JLINKMEM
  // Size of the communication buffer for JLINKMEM
const OS_U32 OS_JLINKMEM_BufferSize = 32u;
#else
const OS_U32 OS_JLINKMEM_BufferSize = 0;     // Communication not used
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
* Function description
*   Returns the current hardware timer count value
*
* Return value
*   Current timer count value
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  return SYS_TICK_VALUE;
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
* Function description
*   Returns if the hardware timer interrupt pending flag is set
*
* Return value
*   == 0; Interrupt pending flag not set
*   != 0: Interrupt pending flag set
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return SYS_INT_CTRL_STATE & (1uL << SYS_PENDSTSET);
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_GetPClkTimer()
*
* Function description
*   Helper function for Percepio embOS-Trace
*/
OS_U32 OS_GetPClkTimer(void) {
  return OS_PCLK_TIMER;
}

/*********************************************************************
*
*       OS_GetTickFreq()
*
* Function description
*   Helper function for Percepio embOS-Trace
*/
OS_U32 OS_GetTickFreq(void) {
  return OS_TICK_FREQ;
}

/*********************************************************************
*
*       OS_Systick()
*
* Function description
*   This is the code that gets called when the processor receives a
*   SysTick exception. SysTick is used as OS timer tick.
*
* NOTES:
*   (1) It has to be inserted in the interrupt vector table, if RAM
*       vectors are not used. Therefore it is declared public
*/
#ifdef __cplusplus
extern "C" {
#endif
void OS_Systick(void);
#ifdef __cplusplus
}
#endif
void OS_Systick(void) {
  OS_EnterNestableInterrupt();
  OS_TICK_Handle();
#if OS_USE_JLINKMEM
  JLINKMEM_Process();
#endif
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  OS_U8  TickPrio;

  //
  // Structure with information about timer frequency, tick frequency, etc.
  // for micro second precise system time.
  //
  OS_SYSTIMER_CONFIG SysTimerConfig = {OS_PCLK_TIMER, OS_TICK_FREQ, 0, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

  OS_IncDI();
  //
  // If not IAR __low_level_init() is not called from startup code
  //
  #ifndef __ICCARM__  // IAR
    __low_level_init();
  #endif
  //
  // Initialize OS timer, clock soure = core clock
  //
  SYS_TICK_RELOAD  = OS_TIMER_RELOAD;
  SYS_TICK_CONTROL = (1uL << SYS_TICK_ENABLE_BIT) | (1uL << SYS_TICK_CLK_SOURCE_BIT);
  //
  // Install Systick Timer Handler and enable timer interrupt
  //
  OS_ARM_InstallISRHandler(OS_ISR_ID_TICK, (OS_ISR_HANDLER*)OS_Systick);
  //
  // Initialize NVIC vector base address. Might be necessary for RAM targets or application not running from 0
  //
  NVIC_VTOR = (OS_U32)&__Vectors;
  //
  // Set the interrupt priority for the system timer to 2nd lowest level to ensure the timer can preempt PendSV handler
  //
  OS_ARM_ISRSetPrio(OS_ISR_ID_TICK, 0xFFu);              // Set to lowest level, ALL BITS set
  TickPrio  = OS_ARM_ISRSetPrio(OS_ISR_ID_TICK, 0xFFu);  // Read priority back to examine relevant preemption-level-bits
  TickPrio -= 1;                                         // Set to low preemption level, 1 level higher than lowest
  OS_ARM_ISRSetPrio(OS_ISR_ID_TICK, TickPrio);
  OS_ARM_EnableISR(OS_ISR_ID_TICK);
  //
  // Setup values for usec precise system time functions
  //
  OS_Config_SysTimer(&SysTimerConfig);
  //
  // Configure and initialize SEGGER SystemView
  //
#if OS_PROFILE
  SEGGER_SYSVIEW_Conf();
#endif

  OS_COM_INIT();
  OS_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggling an output or incrementing a counter)
*/
void OS_Idle(void) {     // Idle loop: No task is ready to execute
  while (1) {
    #if ((OS_USE_JLINKMEM == 0) && (DEBUG == 0))     // Enter CPU halt mode when not in DEBUG build and J-Link communication not used
      #ifdef __ICCARM__  // IAR
        __asm(" wfi");
      #endif
      #ifdef __CC_ARM    // KEIL
        __wfi();
      #endif
      #ifdef __GNUC__    // GCC
        __asm(" wfi");
      #endif
    #endif
  }
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*       This routine is required for task-info via embOSView or high
*       resolution time measurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  OS_U32 Cnt;

  Time = OS_GetTime32();
  Cnt  = (OS_TIMER_RELOAD) - SYS_TICK_VALUE;
  //
  // Check if timer interrupt pending ...
  //
  if (SYS_INT_CTRL_STATE & (1uL << SYS_PENDSTSET)) {
    Cnt = OS_TIMER_RELOAD - SYS_TICK_VALUE;          // Interrupt pending, re-read timer and adjust result
    Time++;
  }
  return (OS_TIMER_RELOAD * Time) + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagnostics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000u);
}

#if OS_UART_USED

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

#define OS_COM_IsrHandler    USART3_IRQHandler

#define USART3_BASE_ADDR     (0x40004800uL)
#define USART3EN_BIT         (18)
#define USART3RST_BIT        (18)
#define GPIOCEN_BIT          (2)

#define GPIOC_BASE_ADDR      (0x40020800uL)
#define GPIOC_MODER          (*(volatile OS_U32*)(GPIOC_BASE_ADDR + 0x00))
#define GPIOC_AFRH           (*(volatile OS_U32*)(GPIOC_BASE_ADDR + 0x24))


/****** MAP selected USART to UART for embOSView ********************/
#define USART_BASE_ADDR      (USART3_BASE_ADDR)
#define RCC_USART_ENR        (RCC_APB1ENR)
#define RCC_USART_RSTR       (RCC_APB1RSTR)
#define USART_GPIO_ENR       (RCC_AHB1ENR)

#define USART_GPIO_ENR_BIT   (GPIOCEN_BIT)
#define USARTxEN_BIT         (USART3EN_BIT)
#define USARTxRST_BIT        (USART3RST_BIT)
#define ISR_ID_OS_UART       (ISR_ID_USART3)

#define USART_SR             (*(volatile OS_U32*)(USART_BASE_ADDR + 0x00))
#define USART_DR             (*(volatile OS_U32*)(USART_BASE_ADDR + 0x04))
#define USART_BRR            (*(volatile OS_U32*)(USART_BASE_ADDR + 0x08))
#define USART_CR1            (*(volatile OS_U32*)(USART_BASE_ADDR + 0x0C))
#define USART_CR2            (*(volatile OS_U32*)(USART_BASE_ADDR + 0x10))
#define USART_CR3            (*(volatile OS_U32*)(USART_BASE_ADDR + 0x14))
#define USART_GTPR           (*(volatile OS_U32*)(USART_BASE_ADDR + 0x18))

//#define GPIOD_CRL            (*(volatile OS_U32*)(0x40011400))

#define US_RXRDY             (0x20)   // RXNE
#define USART_RX_ERROR_FLAGS (0x0F)   // ORE/NE/FE/PE

#define OS_BRR_VALUE         (OS_PCLK_UART / OS_BAUDRATE)


/*********************************************************************
*
*       OS_COM_IsrHandler
*
* Function description
*   The communication interrupt handler for UART communication
*   to embOSView.
*
* NOTES:
*   (1) It has to be inserted in the interrupt vector table, if RAM
*       vectors are not used. Therefore is is declared public
*/
#ifdef __GNUC__
  #pragma GCC diagnostic ignored"-Wunused-but-set-variable" // Avoid variable 'Dummy' set but not used warning
#endif
void OS_COM_IsrHandler(void) {
  volatile OS_U32 Dummy;
  int UsartStatus;

  OS_EnterNestableInterrupt();
  UsartStatus = USART_SR;                        // Examine status register
  do {
    if (UsartStatus & US_RXRDY) {                // Data received?
      if (UsartStatus & USART_RX_ERROR_FLAGS) {  // Any error ?
        Dummy = USART_DR;                        // => Discard data
      } else {
        OS_OnRx(USART_DR);                       // Process actual byte
      }
    }
    else {
      if (OS_OnTx()) {                           // No more characters to send ?
        USART_CR1 &= ~0x40uL;                    // Disable further tx interrupts
      }
    }
    UsartStatus = USART_SR;                      // Examine current status
  }
  while (UsartStatus & US_RXRDY);
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_COM_Send1()
*
*       Sends one character via UART. Never call this from your application
*/
void OS_COM_Send1(OS_U8 c) {
  while (USART_SR & (1 << 7) == 0);
  USART_DR   = (c & (OS_U16)0x01FF);
  USART_CR1 |= 0x40;                           // enable tx interrupt
}

/*********************************************************************
*
*       OS_COM_Init()
*
*       Initialize the selected UART
*       This code is written for USART3 on the STM3220F-Eval board
*       and will have to be modified when an other USART is used.
*/
void OS_COM_Init(void) {
  OS_U8 Priority;

  //
  // Enable clock for GPIO used for UART
  //
  USART_GPIO_ENR |=  (1uL << USART_GPIO_ENR_BIT);    // Enable clock for GPIO used for USART
  //
  // Setup mode for PIO used for USART 3, has to modified when a different USART is used
  //
  GPIOC_MODER &= ~((3uL << (10 * 2)) | (3uL << (11 * 2)));  // Clear Mode for Rx and Tx pin
  GPIOC_MODER |=  ((2uL << (10 * 2)) | (2uL << (11 * 2)));  // Set alternate function mode for Rx and Tx pin
  GPIOC_AFRH  &= ~((0x0FuL << 12) | (0x0FuL << 8));
  GPIOC_AFRH  |=  ((0x07uL << 12) | (0x07uL << 8));
  //
  // Enable clock for UART and clear peripheral reset
  //
  RCC_USART_ENR  |=  (1uL << USARTxEN_BIT);          // Enable USART clock
  RCC_USART_RSTR &= ~(1uL << USARTxRST_BIT);         // Clear USART reset
  //
  // Setup the UASRT used for embOSView
  //
  USART_CR2   = 0x00            // b3..b0:   Addr in multiprocessor communication, don't care
              | (0uL << 4)      // b4:       Reserved, always 0
              | (0uL << 5)      // b5:       Lin break detection, don't care
              | (0uL << 6)      // b6:       Lin break detection interrupt enable, off
              | (0uL << 7)      // b7:       Reserved, always 0
              | (0uL << 8)      // b8:       LBCL, Last bit clock pulse, don't care in async mode
              | (0uL << 9)      // b9:       CPHA, Clock phase, don't care in async mode
              | (0uL << 10)     // b10:      CPOL, Clock polarity, don't care in async mode
              | (0uL << 11)     // b11:      CLKEN, Clock enable on CK pin, not used in async mode
              | (0uL << 12)     // b13..b12: STOP, 0 => 1 Stop bit
              | (0uL << 14)     // b14:      LINEN, Linmode disabled for async mode
              | (0uL << 15)     // B31..b15: Reserved, always 0
              ;

  USART_CR1   = (0uL << 0)      // b0:       SBK, send break, disabled
              | (0uL << 1)      // b1:       RWU, receiver wakeup, 0 => Receiver in normal mode
              | (1uL << 2)      // b2:       RE, receiver enabled
              | (1uL << 3)      // b3:       TE, transmitter enabled
              | (0uL << 4)      // b4:       IDLEIE, Idle interrupt, disabled
              | (1uL << 5)      // b5:       RXNEIE, Rx interrupt enabled
              | (0uL << 6)      // b6:       TCIE, transmission complete interrupt, disabled
              | (0uL << 7)      // b7:       TXEIE, Transmitter empty interrupt, disabled
              | (0uL << 8)      // b8:       PE, Parity error interrupt, disabled
              | (0uL << 9)      // b9:       PS, Parity selection, Even parity
              | (0uL << 10)     // b10:      PCE, Parity control enable, disabled
              | (0uL << 11)     // b11:      WAKE, Wakeup method, don't care
              | (0uL << 12)     // b12:      M, word length, 0 => 8 data bits
              | (0uL << 13)     // b13:      UE, USART enbale, disaled by now
              | (0uL << 14)     // B31..b14: Reserved, always 0
              ;

  USART_CR3   = 0x00;
  USART_BRR   = OS_BRR_VALUE;   // Set calculated baud rate without fraction
  USART_CR1  |= (1uL << 13);    // Enable uart
  //
  // Install USART2 Handler with preemtion level one above lowest level to ensure communication
  //
  OS_ARM_InstallISRHandler(ISR_ID_OS_UART, (OS_ISR_HANDLER*)OS_COM_IsrHandler);
  OS_ARM_ISRSetPrio(ISR_ID_OS_UART, 0xFFu);              // Set lowest Priority, ALL BITS set
  Priority  = OS_ARM_ISRSetPrio(ISR_ID_OS_UART, 0xFFu);  // Read back priority to examine preemption level bits
  Priority -= 1;                                         // Set priority one level above lowest priority
  OS_ARM_ISRSetPrio(ISR_ID_OS_UART, Priority);
  OS_ARM_EnableISR(ISR_ID_OS_UART);
}

#elif OS_USE_JLINKMEM

/*********************************************************************
*
*       _JLINKMEM_OnRx()
*/
static void _JLINKMEM_OnRx(OS_U8 Data) {
  OS_OnRx(Data);
}

/*********************************************************************
*
*       _JLINKMEM_OnTx()
*/
static void _JLINKMEM_OnTx(void) {
  OS_OnTx();
}

/*********************************************************************
*
*       _JLINKMEM_GetNextChar()
*/
static OS_INT _JLINKMEM_GetNextChar(void) {
  return OS_COM_GetNextChar();
}

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize memory access for embOSView
*/
void OS_COM_Init(void) {
  JLINKMEM_SetpfOnRx(_JLINKMEM_OnRx);
  JLINKMEM_SetpfOnTx(_JLINKMEM_OnTx);
  JLINKMEM_SetpfGetNextChar(_JLINKMEM_GetNextChar);
}

/*********************************************************************
*
*       OS_COM_Send1()
*       Send one character via memory
*/
void OS_COM_Send1(OS_U8 c) {
  JLINKMEM_SendChar(c);
}

#else // No UART communication for OS enabled, define dummy functions

/*********************************************************************
*
*       Communication for embOSView not selected
*
**********************************************************************
*/
void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* Avoid compiler warning */
  OS_COM_ClearTxActive();  /* Let the OS know that Tx is not busy */
}
#endif /*  OS_UART_USED  */

/*********************************************************************
*
*       OS interrupt handler and ISR specific functions
*
**********************************************************************
*/

#if OS_USE_VARINTTABLE
  //
  // The interrupt vector table may be located anywhere in RAM
  //
  #ifdef __ICCARM__  // IAR
    #pragma data_alignment=512
    __no_init void (*g_pfnRAMVectors[NUM_INTERRUPTS]) (void);
  #endif  // __ICCARM__

  #ifdef __CC_ARM    // KEIL
    __attribute__ (zero_init, aligned(512)) void (*g_pfnRAMVectors[NUM_INTERRUPTS])(void);
  #endif

  #ifdef __GNUC__    // GCC
    void (*g_pfnRAMVectors[NUM_INTERRUPTS]) (void) __attribute__ ((aligned (512)));
  #endif
#endif

/*********************************************************************
*
*       OS_ARM_InstallISRHandler
*/
OS_ISR_HANDLER* OS_ARM_InstallISRHandler (int ISRIndex, OS_ISR_HANDLER* pISRHandler) {
#if OS_USE_VARINTTABLE
  OS_ISR_HANDLER*  pOldHandler;
  OS_U32           ulIdx;
  OS_U32*          pVect;

  pOldHandler = NULL;
  //
  // Check whether the RAM vector table has been initialized.
  //
  if ((*(OS_U32*)NVIC_VTOREG_ADDR) != (unsigned long)g_pfnRAMVectors) {
    //
    // Copy the vector table from the beginning of FLASH to the RAM vector table.
    //
    pVect = (OS_U32*)(*(OS_U32*)NVIC_VTOREG_ADDR);
    for(ulIdx = 0; ulIdx < NUM_INTERRUPTS; ulIdx++) {
      g_pfnRAMVectors[ulIdx] = (void (*)(void))(pVect[ulIdx]);
    }
    //
    // Program the NVIC vector base register to point at the RAM vector table.
    //
    *(OS_U32*)NVIC_VTOREG_ADDR = (OS_U32)g_pfnRAMVectors;
  }
  //
  // Save the interrupt handler.
  //
  pOldHandler = g_pfnRAMVectors[ISRIndex];
  g_pfnRAMVectors[ISRIndex] = pISRHandler;
  return (pOldHandler);
#else
  //
  // The function does nothing if vector table is constant
  //
  OS_USEPARA(ISRIndex);
  OS_USEPARA(pISRHandler);
  return (NULL);
#endif
}

/*********************************************************************
*
*       OS_ARM_EnableISR
*/
void OS_ARM_EnableISR(int ISRIndex) {
  OS_DI();
  if (ISRIndex < NUM_INTERRUPTS) {
    if (ISRIndex >= 16) {
      //
      // Enable standard "external" interrupts, starting at index 16
      //
      ISRIndex -= 16;
      *(((OS_U32*) NVIC_ENABLE_ADDR) + (ISRIndex >> 5)) = (1uL << (ISRIndex & 0x1Fu));
    } else if (ISRIndex == ISR_ID_MPU) {
      //
      // Enable the MemManage interrupt.
      //
      SYSHND_CTRL |= NVIC_SYS_HND_CTRL_MEM;
    } else if (ISRIndex == ISR_ID_BUS) {
      //
      // Enable the bus fault interrupt.
      //
      SYSHND_CTRL |= NVIC_SYS_HND_CTRL_BUS;
    } else if (ISRIndex == ISR_ID_USAGE) {
      //
      // Enable the usage fault interrupt.
      //
      SYSHND_CTRL |= NVIC_SYS_HND_CTRL_USAGE;
    } else if (ISRIndex == ISR_ID_SYSTICK) {
      //
      // Enable the System Tick interrupt.
      //
      SYS_TICK_CONTROL |= (1 << SYS_TICK_INT_ENABLE_BIT);
    }
  }
  OS_RestoreI();
}

/*********************************************************************
*
*       OS_ARM_DisableISR
*/
void OS_ARM_DisableISR(int ISRIndex) {
  OS_DI();
  if (ISRIndex < NUM_INTERRUPTS) {
    if (ISRIndex >= 16) {
      //
      // Disable standard "external" interrupts
      //
      ISRIndex -= 16;
      *(((OS_U32*) NVIC_DISABLE_ADDR) + (ISRIndex >> 5)) = (1uL << (ISRIndex & 0x1Fu));
    } else if (ISRIndex == ISR_ID_MPU) {
      //
      // Disable the MemManage interrupt.
      //
      SYSHND_CTRL &= ~NVIC_SYS_HND_CTRL_MEM;
    } else if (ISRIndex == ISR_ID_BUS) {
      //
      // Disable the bus fault interrupt.
      //
      SYSHND_CTRL &= ~NVIC_SYS_HND_CTRL_BUS;
    } else if (ISRIndex == ISR_ID_USAGE) {
      //
      // Disable the usage fault interrupt.
      //
      SYSHND_CTRL &= ~NVIC_SYS_HND_CTRL_USAGE;
    } else if (ISRIndex == ISR_ID_SYSTICK) {
      //
      // Enable the System Tick interrupt.
      //
      SYS_TICK_CONTROL &= ~(1uL << SYS_TICK_INT_ENABLE_BIT);
    }
  }
  OS_RestoreI();
}

/*********************************************************************
*
*       OS_ARM_ISRSetPrio
*
*   Notes:
*     (1) Some priorities of system handler are reserved
*         0..3 : Priority can not be set
*         7..10: Reserved
*         13   : Reserved
*     (2) System handler use different control register. This affects
*         ISRIndex 0..15
*/
int OS_ARM_ISRSetPrio(int ISRIndex, int Prio) {
  OS_U8* pPrio;
  int    OldPrio;

  OldPrio = 0;
  if (ISRIndex < NUM_INTERRUPTS) {
    if (ISRIndex >= 16) {
      //
      // Handle standard "external" interrupts
      //
      ISRIndex -= 16;                   // Adjust index
      OS_DI();
      pPrio = (OS_U8*)(NVIC_PRIOBASE_ADDR + ISRIndex);
      OldPrio = *pPrio;
      *pPrio = Prio;
      OS_RestoreI();
    } else {
      //
      // Handle System Interrupt controller
      //
      if ((ISRIndex < 4) | ((ISRIndex >= 7) && (ISRIndex <= 10)) | (ISRIndex == 13)) {
        //
        // Reserved ISR channel, do nothing
        //
      } else {
        //
        // Set priority in system interrupt priority control register
        //
        OS_DI();
        pPrio = (OS_U8*)(SYSPRI1_ADDR);
        ISRIndex -= 4;                  // Adjust Index
        OldPrio = pPrio[ISRIndex];
        pPrio[ISRIndex] = Prio;
        OS_RestoreI();
      }
    }
  }
  return OldPrio;
}

/*********************************************************************
*
*       __low_level_init()
*
*/
#ifdef __cplusplus
extern "C" {
#endif
int __low_level_init(void);    // Avoid "no prototype" warning
#ifdef __cplusplus
  }
#endif
int __low_level_init(void) {
  volatile OS_U32 Dummy;
  OS_U32          RegVal;

  //
  // Setup the clock system, reset all peripheral clock system registers to default (reset) state
  //
  RCC_AHB1ENR   = 0;
  RCC_AHB2ENR   = 0;
  RCC_AHB3ENR   = 0;

  RCC_APB1ENR   = 0;
  RCC_APB2ENR   = 0;

  RCC_AHB1LPENR = 0x7E6791FFuL;   // Default value after reset as listed in reference manual
  RCC_AHB2LPENR = 0x000000F1uL;   // Default value after reset as listed in reference manual
  RCC_AHB3LPENR = 0x00000001uL;   // Default value after reset as listed in reference manual

  RCC_APB1LPENR = 0x36FEC9FFuL;   // Default value after reset as listed in reference manual
  RCC_APB2LPENR = 0x00075F33uL;   // Default value after reset as listed in reference manual
  //
  // Resets the RCC clock configuration to the default reset state.
  //
  RCC_CR       |= (1uL << RCC_CR_HSION_BIT);  // Set HSION bit

  RCC_CFGR     &= 0xFF000000uL;               // Reset SW[1:0] => use internal HSI, reset HPRE, PPRE1, PPRE2, RTCPRE and MCO bits
  RCC_CIR       = 0x0;                        // Disable all interrupts
  //
  // Configure HSE
  //
  RCC_CR      &= ~(1uL << RCC_CR_HSEON_BIT);  // Clear HSEON
  RCC_CR      &= ~(1uL << RCC_CR_HSEBYP_BIT); // Clear HSEBYP
  RCC_CR      |=  (1uL << RCC_CR_HSEON_BIT);  // Enable HSE
  while ((RCC_CR & (1uL << RCC_CR_HSERDY_BIT)) == 0) {
    //
    // Wait until HSE is ready
    //
  }
  //
  // Initialize Flash, 2 wait states and Prefetch enable
  //
  FLASH_ACR   &= ~ACR_LATENCY_MASK;           // Clear Waitstates
  if ((FLASH_ACR & (1uL << FLASH_ACR_ICEN_BIT)) == 0) {   // Reset Instruction cache if not enabled
    FLASH_ACR |= (1uL << FLASH_ACR_ICRST_BIT);
  }
  if ((FLASH_ACR & (1uL << FLASH_ACR_DCEN_BIT)) == 0) {   // Reset Data cache if not enabled
    FLASH_ACR |= (1uL << FLASH_ACR_DCRST_BIT);
  }
  FLASH_ACR   &= ~((1uL << FLASH_ACR_ICRST_BIT) | (1uL << FLASH_ACR_DCRST_BIT));  // Clear cache reset
  FLASH_ACR   &= ~(1uL << FLASH_ACR_PRFTEN_BIT);          // Clear Prefetch unit
  FLASH_ACR   |= (FLASH_WS << FLASH_ACR_LATENCY_BIT);     // Set Flash Wait states (according to CPU frequency)
  //
  // Check that Flash wait states are really set (Chapter 2.3.4 of the referenc manual)
  //
  while ((FLASH_ACR & ACR_LATENCY_MASK) != (FLASH_WS << FLASH_ACR_LATENCY_BIT)) {
    FLASH_ACR   |= (FLASH_WS << FLASH_ACR_LATENCY_BIT);   // Set Flash Wait states (according to CPU frequency)
  }
  FLASH_ACR   |= (1uL << FLASH_ACR_PRFTEN_BIT);           // Activate prefetch unit
  //
  // Enable the caches
  //
  FLASH_ACR   |= ((1uL << FLASH_ACR_ICEN_BIT) | (1uL << FLASH_ACR_DCEN_BIT));

  //
  // Initialize PLL to speed up the system
  //
  //
  // HCLK = SYSCLK
  //
  RegVal   = RCC_CFGR;
  RegVal  &= ~(0x0FuL << RCC_CFGR_HPRE_BIT);    // 0xx: Clear HCLK divider
  RCC_CFGR = RegVal;
  //
  // PCLK2 = HCLK / 2
  //
  RegVal   = RCC_CFGR;
  RegVal  &= ~(0x07uL << RCC_CFGR_PPRE2_BIT);      // 0xx: Clear PRE2 (APB2) prescaler
  RegVal  |=  (PRE2_DIV2 << RCC_CFGR_PPRE2_BIT);   // Set APB2 prescaler, divide by 2, ensures APB2 clock <= 60 MHz
  RCC_CFGR = RegVal;
  //
  // PCLK1 = HCLK / 4
  //
  RegVal   = RCC_CFGR;
  RegVal  &= ~(0x07uL << RCC_CFGR_PPRE1_BIT);      // 0xx: Clear PRE1 (APB1) prescaler
  RegVal  |=  (PRE1_DIV4 << RCC_CFGR_PPRE1_BIT);   // Set APB1 prescaler, divide by 4, ensures APB1 clock <= 30 MHz
  RCC_CFGR = RegVal;
  //
  // Clear (and optionally set) Clock prescaler for RTC clock
  //
  RegVal   = RCC_CFGR;
  RegVal  &= ~(0x1FuL << RCC_CFGR_RTCPRE_BIT);     // 0xx: Clear RTC prescaler => no clock from HSE
  RCC_CFGR = RegVal;

#if RTC_USE_HSE
  RegVal   = RCC_CFGR;
  RegVal  |= (HSE_CLOCK << RCC_CFGR_RTCPRE_BIT);   // Set RTC prescaler: RTC input has to be 1MHz
  RCC_CFGR = RegVal;
#endif
  //
  // Configure MCO output and prescaler (if required)
  //

  //
  // Setup PLL, Cortex M3 clock according to OS_FSYS and 48 MHz USB OTG FS clock
  //
#if (OS_FSYS == 120000000uL)
  RegVal      = (HSE_CLOCK << RCC_CFGR_PLLM_BIT)          // Set the input divider to feed 1 MHz into the PLL (required)
              | (240 << RCC_CFGR_PLLN_BIT) // Set the multiplier, generate 240 MHz VCO clock (192 <= fVCO <= 432)
              | (  0 << RCC_CFGR_PLLP_BIT)                // PLL output divider = 2 => System clock = 120 MHz
              | (  5 << RCC_CFGR_PLLQ_BIT)                // PLL USB output divider = 5 => USB OTG FS clock = 48 MHz
              | (  2 << RCC_CFGR_PLLR_BIT)                // PLL I2S clock output divider = 2 => I2Sclk = 120 MHz
              | (  1 << RCC_CFGR_PLLSRC_BIT)              // 1 => HSE is clock source for PLL
              ;
#elif (OS_FSYS == 96000000uL)
  RegVal      = (HSE_CLOCK << RCC_CFGR_PLLM_BIT)          // Set the input divider to feed 1 MHz into the PLL (required)
              | (192 << RCC_CFGR_PLLN_BIT)                // Set the multiplier, generate 192 MHz VCO clock (192 <= fVCO <= 432)
              | (  0 << RCC_CFGR_PLLP_BIT)                // PLL output divider = 2 => System clock = 92 MHz
              | (  4 << RCC_CFGR_PLLQ_BIT)                // PLL USB output divider = 4 => USB OTG FS clock = 48 MHz
              | (  2 << RCC_CFGR_PLLR_BIT)                // PLL I2S clock output divider = 2 => I2Sclk = 96 MHz
              | (  1 << RCC_CFGR_PLLSRC_BIT)              // 1 => HSE is clock source for PLL
              ;
#else
  #error "PLL inititialization has to be modified"
#endif

  RCC_PLLCFGR = RegVal;                                   // Setup the configuration data
  //
  // Switch on PLL and wait until PLL is ready
  //
  RCC_CR     |= (1uL << RCC_CR_PLLON_BIT);
  while ((RCC_CR & (1ul << RCC_CR_PLLRDY_BIT)) == 0) {
    //
    // Wait until PLL is ready
    //
  }
  //
  // Add a short delay. Required!
  //
  for (Dummy = 0; Dummy < 1000; Dummy++) {
  }
  //
  // Select PLL as system clock source
  //
  RegVal    = RCC_CFGR;
  RegVal   &= ~(0x03uL << RCC_CFGR_SW_BIT);
  RegVal   |= (0x02uL << RCC_CFGR_SW_BIT);          // 0x02: PLL selected as system clock
  RCC_CFGR  = RegVal;
  while ((RCC_CFGR & (0x3uL << RCC_CFGR_SWS_BIT)) != (0x02uL << RCC_CFGR_SWS_BIT)) {
    //
    // Wait until PLL is used as system clock source
    //
  }
  //
  // Initialize NVIC vector base address. Might be necessary for RAM targets or application not running from 0
  //
  NVIC_VTOR = (OS_U32)&__Vectors;
  //
  // Initalize other peripherals, memories, if required
  //

  return 1;  // Always return 1 to tell the startup code to initialize the data segments
}

/****** Final check of configuration ********************************/

#ifndef OS_UART_USED
  #error "OS_UART_USED has to be defined"
#endif

/****** End Of File *************************************************/
